/*
 * Copyright: Scheer PAS GmbH
 */
package com.scheer.pas.large;

import ch.e2e.bridge.server.XumlLibraryClass;

import java.util.NoSuchElementException;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

/**
 * This example shows how you can reduce the amount of data to be transferred between the xUML Runtime and the JVM
 * when dealing with large objects.
 */

@SuppressWarnings("unused")
@XumlLibraryClass(name = "LargeObject")
public class LargeObjectHandling {
    private static final ConcurrentMap<String, LargeObject> MAP = new ConcurrentHashMap<String, LargeObject>();

    /**
     * Creates a new instance of <code>LargeObject</code> and returns its id. The other methods
     * of <code>Operations</code> use this id to refer to this instance.
     *
     * @return Returns the id of the new <code>LargeObject</code>.
     * @throws IllegalStateException
     */
    public static String newLargeObject() {
        LargeObject largeObject = new LargeObject();
        String id = largeObject.getId();
        if (MAP.putIfAbsent(id, largeObject) != null) {
            throw new IllegalStateException();
        }
        return id;
    }

    /**
     * Gets the value of list element <code>index</code>.
     *
     * @param id Provide the id of the <code>LargeObject</code>.
     * @param index Provide the index of the <code>LargeObject</code> element to return.
     * @return Returns the <code>LargeObject</code> element at the specified position of the list.
     * @throws IndexOutOfBoundsException if the index is out of range
     * @throws NoSuchElementException if no <code>LargeObject</code> with the provided <code>id</code> exists
     */
    public static String getValue(String id, int index) {
        return getLargeObject(id).getValue(index);
    }

    /**
     * Sets the value of list element <code>index</code>.
     *
     * @param id Provide id of the <code>LargeObject</code>.
     * @param index Provide the index of the list element to be set.
     * @param value Provide the value to be inserted.
     * @throws IndexOutOfBoundsException if the index is out of range
     * @throws NoSuchElementException if no <code>LargeObject</code> with Id <code>id</code> exists
     */
    public static void setValue(String id, int index, String value) {
        getLargeObject(id).setValue(index, value);
    }

    /**
     * Processes the <code>LargeObject</code> identified by <code>id</code>.
     *
     * @param id Provide the id of the <code>LargeObject</code> to be processed.
     * @throws NoSuchElementException if no <code>LargeObject</code> with Id <code>id</code> exists
     */
    public static void process(String id) {
        LargeObject largeObject = getLargeObject(id);
        // Implement some processing of the LargeObject here.
    }

    /**
     * Removes the <code>LargeObject</code> referenced by <code>id</code> from the internal cache.
     *
     * @param id Provide the id of the <code>LargeObject</code> to be removed from the cache.
     */
    public static void releaseLargeObject(String id) {
        MAP.remove(id);
    }

    /**
     * Returns the <code>LargeObject</code> referenced by <code>id</code>.
     *
     * @param id Provide the id of the <code>LargeObject</code>.
     * @return Returns the <code>LargeObject</code> referenced by <code>id</code>.
     * @throws NoSuchElementException if no <code>LargeObject</code> with Id <code>id</code> exists
     */
    private static LargeObject getLargeObject(String id) {
        LargeObject largeObject = MAP.get(id);
        if (id == null) {
            throw new NoSuchElementException();
        }
        return largeObject;
    }

}
